#!/usr/share/ucs-test/runner /usr/bin/pytest -l -v -s
## -*- coding: utf-8 -*-
## desc: move OX user from one context into another
## tags: [apptest]
## roles: [domaincontroller_master,domaincontroller_backup]
## exposure: dangerous
## packages:
##   - univention-ox (>= 8.1.0-72)
##   - open-xchange-authentication-ucs (>= 7.8.4-0)
## bugs: [43264, 52357]

import pytest
import univention.config_registry
import univention.testing.udm as testing_udm
import univention.testing.ucr as testing_ucr
import univention.testing.strings as uts
import univention.testing.utils as utils
import essential.common


@pytest.mark.parametrize("copy_user_data_to_new_context", ["yes", "no"])
def test_change_context(copy_user_data_to_new_context):
	with essential.common.restart_listener():
		univention.config_registry.handler_set(['ox/listener/copy-user-data-to-new-context={}'.format(copy_user_data_to_new_context)])

	with essential.common.OXTestEnv() as oxenv, \
			testing_udm.UCSTestUDM() as udm, \
			testing_ucr.UCSTestConfigRegistry() as ucr, \
			essential.common.OXTestEmptyQueue():

		context_id_A, context_id_B = oxenv.create_contexts(2)

		oxenv.check_if_context_exists_in_ldap(context_id_A)
		oxenv.check_if_context_exists_in_ox_via_csv(context_id_A)

		oxenv.check_if_context_exists_in_ldap(context_id_B)
		oxenv.check_if_context_exists_in_ox_via_csv(context_id_B)

		print('*** Creating user in OX context {}...'.format(context_id_A))
		mail_domain = ucr.get('mail/hosteddomains').split()[-1].strip()
		ox_user_password = uts.random_string()
		ox_mail = '{}@{}'.format(uts.random_username(), mail_domain)
		ox_user_dn, ox_username = udm.create_user(
			mailPrimaryAddress=ox_mail,
			password=ox_user_password,
			isOxUser='OK',
			oxContext=context_id_A)
		utils.wait_for_replication()

		oxenv.check_if_user_exists_in_context(ox_username, context_id_A)  # should exist in A
		oxenv.check_if_user_exists_in_context(ox_username, context_id_B, expected=False)  # should NOT exist in B
		oxenv.check_if_user_can_login(ox_username, ox_user_password)

		print('*** Moving user {} from context {} to context {}'.format(ox_username, context_id_A, context_id_B))
		udm.modify_object('users/user', dn=ox_user_dn, oxContext=context_id_B)
		utils.wait_for_replication()
		copy_user_data_to_new_context = copy_user_data_to_new_context == 'yes'
		oxenv.check_if_user_exists_in_context(ox_username, context_id_B) # should exist in B
		oxenv.check_if_user_exists_in_context(ox_username, context_id_A, expected=False)  # should NOT exist in A
		#		if copy_user_data_to_new_context:
		oxenv.check_if_user_can_login(ox_username, ox_user_password)
		# i have to check the user-data otherwise.
		exists1 = oxenv.check_if_resource_exists_via_csv(context_id_B, ox_username, raise_exception=False)

